"""
This module provides four plot engines (matplotlib, plotly, gnuplot, and plotext) and several type of plots, including scatter, line, heatmap, and so on. 

This package is provided to you under GPLv3 License.
"""