"""
 *******           **                  **********               **        
/**////**         /**          **   **/////**///               /**        
/**    /** **   **/** **    **//** **     /**  ******   ****** /**  ******
/**    /**/**  /**/**/**   /** //***      /** **////** **////**/** **//// 
/**    /**/**  /**/**//** /**   /**       /**/**   /**/**   /**/**//***** 
/**    ** /**  /**/** //****    **        /**/**   /**/**   /**/** /////**
/*******  //******/**  //**    **         /**//****** //****** *** ****** 
///////    ////// //    //    //          //  //////   ////// /// //////  

DuIvyTools (v0.5.0) is a simple analysis and visualization tool for GROMACS result files, written by 杜艾维 (https://github.com/CharlesHahn/DuIvyTools). 

DuIvyTools contains three modules: `Commands` for providing several commands to deal with data and visualization, `FileParser` for providing parser of several type of files (.gro, .mdp, .ndx, .pdb, .xpm, .xvg), `Visualizer` for providing several visualization class, including four plot engine (matplotlib, plotly, gnuplot, and plotext) and several type of plots.

This package is provided to you under GPLv3 License.
"""