"""
 *******           **                  **********               **        
/**////**         /**          **   **/////**///               /**        
/**    /** **   **/** **    **//** **     /**  ******   ****** /**  ******
/**    /**/**  /**/**/**   /** //***      /** **////** **////**/** **//// 
/**    /**/**  /**/**//** /**   /**       /**/**   /**/**   /**/**//***** 
/**    ** /**  /**/** //****    **        /**/**   /**/**   /**/** /////**
/*******  //******/**  //**    **         /**//****** //****** *** ****** 
///////    ////// //    //    //          //  //////   ////// /// //////  

DuIvyTools is a simple analysis and visualization tool for GROMACS result files
written by 杜艾维 (https://github.com/CharlesHahn/DuIvyTools). 

DuIvyTools package contains two DuIvyTools actually now. One is `DuIvyTools` which is the newest DuIvyTools v0.5 and noted as `dit`, the other is the `DuIvyTools_old` which is v0.4.8 and noted as `dito`.
Since reconstruction were done for v0.5.0 and some features were dropped, the v0.4.8 were kept in package to keep integrity temporarily. 

This package is provided to you under GPLv3 License.
"""