"""
DuIvyTools is a simple analysis and visualization tool for GROMACS result files.
written by CharlesHahn (https://github.com/CharlesHahn/DuIvyTools). 

THIS PACKAGE (`DuIvyTools_old`) contains the content of the DuIvyTools v0.4.8. 

Python modules
----------------
The package consists of the following Python modules:
* DIT : the CLI application.
* XPM : module to process XPM files.
* XVG : module to process XVG files.
* NDX : module to process NDX files.
* MDP : module to generate MDP files.
* HELP : module to provide help messages.
* FindCenter : module to find center atom of one gro file
* PipiDistAng : module to calculate the distance and angle of two pi-ring
* HydrogenBond: module to process hbond information generated by `gmx hbond`
* MolMap : module to map coordinates from one pdb file to another pdb file
* DCCM : module to generate dynamic cross-correlation map from ascii file 
* MplStyle: module to generate mplstyle template in current folder

This file is provided to you under GPLv3 License.
"""
